/*
	This file is part of FireBible, a bible study tool for Firefox.
	Copyright (C) 2008 Brian Fernandes <infernalproteus@gmail.com>

    FireBible is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/
const BIBLEPROT_HANDLER_CONTRACTID = "@mozilla.org/network/protocol;1?name=bible";
const BIBLEPROT_HANDLER_CID = Components.ID("{5b44bfa6-a70b-11dc-8314-0800200c9a66}");
const SWORDPROT_HANDLER_CONTRACTID = "@mozilla.org/network/protocol;1?name=sword";
const SWORDPROT_HANDLER_CID = Components.ID("{20e52130-2400-11dd-bd0b-0800200c9a66}");
const STANDARDURL_CONTRACTID = "@mozilla.org/network/standard-url;1";
const STRING_IS_CONTRACTID = "@mozilla.org/io/string-input-stream;1";
    

const nsISupports        = Components.interfaces.nsISupports;
const nsIProtocolHandler = Components.interfaces.nsIProtocolHandler;
const nsIStandardURL     = Components.interfaces.nsIStandardURL;
const nsIURI             = Components.interfaces.nsIURI;
const nsIChannel         = Components.interfaces.nsIChannel;
const nsIRequest         = Components.interfaces.nsIRequest;
const nsIInputStream 	 = Components.interfaces.nsIInputStream;
const nsIStringInputStream = Components.interfaces.nsIStringInputStream;

function BibleProtocolHandler() {
	this.wrappedJSObject = this;
	this.windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);	
};

BibleProtocolHandler.prototype.protocolFlags = nsIProtocolHandler.URI_NORELATIVE | 
												nsIProtocolHandler.URI_NOAUTH |
												nsIProtocolHandler.URI_IS_UI_RESOURCE;

if ("URI_LOADABLE_BY_ANYONE" in nsIProtocolHandler) {
  BibleProtocolHandler.prototype.protocolFlags |=
      nsIProtocolHandler.URI_LOADABLE_BY_ANYONE;
}

BibleProtocolHandler.prototype.allowPort = function (port, schem) {
	return false;
};
	
BibleProtocolHandler.prototype.newURI = function(spec, charset, baseURI) {
	var cls = Components.classes[STANDARDURL_CONTRACTID];
   	var url = cls.createInstance(nsIStandardURL);
   	url.init(nsIStandardURL.URLTYPE_STANDARD, 80, spec, charset, baseURI);
   	return url.QueryInterface(nsIURI);
};
	
BibleProtocolHandler.prototype.newChannel = function(URI) {
		if (this.jsword == null || !this.jsword.utilsInitialized) {
			var win = this.windowManager.getMostRecentWindow("navigator:browser");
			win.fireBible.initFromProtocol();
		}
		this.jsword.log("new channel");
		return new BibleChannel(URI, this.jsword, this.windowManager);		
	};
	
BibleProtocolHandler.prototype.getJSword = function() {
	return this.jsword;
}

BibleProtocolHandler.prototype.setJSword = function(jsword) {
	this.jsword = jsword;
}

BibleProtocolHandler.prototype.getLocation = function() {
	return __LOCATION__;
}
			
BibleProtocolHandler.prototype.QueryInterface = function(aIID) {
	if (!aIID.equals(nsIProtocolHandler) &&    
        !aIID.equals(nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
	
};

function FireBibleFactory(protocol) {
	this.protocol = protocol;	
};

FireBibleFactory.prototype.createInstance = function(aOuter, aIID) {
	if (aOuter != null) {
		throw Components.results.NS_ERROR_NO_AGGREGATION;
	}
	
	if (aIID.equals(nsIProtocolHandler) || aIID.equals(nsISupports)) {
		var protHandler = new BibleProtocolHandler();
		protHandler.schem = this.protocol;
		return protHandler;
	} 
	
	throw Components.results.NS_ERROR_INVALID_ARG;
};

var FireBibleModule = {
	registerSelf: function(aCompMgr, aFileSpec, aLocation, aType) {
		var compMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
		
		compMgr.registerFactoryLocation(BIBLEPROT_HANDLER_CID,
                                    "bible protocol handler",
                                    BIBLEPROT_HANDLER_CONTRACTID,
                                    aFileSpec, aLocation, aType);
                                    
		compMgr.registerFactoryLocation(SWORDPROT_HANDLER_CID,
                                    "sword protocol handler",
                                    SWORDPROT_HANDLER_CONTRACTID,
                                    aFileSpec, aLocation, aType);                                    
    
	},
	
	unregisterSelf: function(aCompMgr, aCID, aIID) {
		var compMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    	compMgr.unregisterFactoryLocation(BIBLEPROT_HANDLER_CID, aLocation);
    	compMgr.unregisterFactoryLocation(SWORDPROT_HANDLER_CID, aLocation);    
	},
	
	getClassObject: function(aCompMgr, aCID, aIID) {
		if (!aIID.equals(Components.interfaces.nsIFactory))
      		throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    	if (aCID.equals(BIBLEPROT_HANDLER_CID)) {
      		return new FireBibleFactory("bible");
      	}
      	
      	if (aCID.equals(SWORDPROT_HANDLER_CID)) {
      		return new FireBibleFactory("sword");
      	}      	

    	throw Components.results.NS_ERROR_NO_INTERFACE;
	},
	
	canUnload: function() {
		return true;
	}
};

function BibleChannel(uri, jsword, wm) {
	this.URI = uri;
	this.originalURI = uri;
	this._isPending = false;
	this.contentCharset = "UTF-8";
	this.jsword = jsword;
	this.windowManager = wm;
};


BibleChannel.prototype = {
	loadAttributes: null,
	contentType: "text/html",
	contentLength: -1,
	owner: null,
	loadGroup: null,
	notificationCallbacks: null,
	securityInfo: null,
	status: Components.results.NS_OK,
	
	cancel: function(status) {
		this.jsword.log("cancel");
	},
	
	isPending: function() {
		this.jsword.log("pending");
		return this._isPending;
	},
	
	suspend: function() {
		this.jsword.log("suspend " + this.notificationCallbacks);
		//this.loadGroup.removeRequest(this, null, Components.results.NS_OK);
		throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
	},
	
	resume: function() {
		this.jsword.log("resume");
		throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
	},
	
	QueryInterface: function (iId) {
		if (!iId.equals(nsIChannel) && !iId.equals(nsIRequest) 
			&& !iId.equals(nsISupports)) {
			throw Components.results.NS_ERROR_NO_INTERFACE;
		}
		return this;		
	},
	
	asyncOpen: function(observer, context) {
		this._isPending = true;
		
		try {
			if (this.loadGroup) {
				this.loadGroup.addRequest(this, null);
			}
			
			var html = this.getHTML();
			observer.onStartRequest(this, context);
			var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
							.createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
			converter.charset = "UTF-8";
			var bst = converter.convertToInputStream(html);
			var len = bst.available();			
			observer.onDataAvailable(this, context, bst, 0, len);						
			//bst.close();
			observer.onStopRequest(this, context, Components.results.NS_OK);
			if (this.loadGroup) {
        		this.loadGroup.removeRequest (this, null, Components.results.NS_OK);
        	}
			this._isPending = false;
					
		} catch (e) {
			this.jsword.log("error " + e);
		}
	},
	
	open: function() {
		throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
	},
	
	getHTML: function() {
		var win = this.windowManager.getMostRecentWindow("navigator:browser");
		return win.fireBible.load(this.URI);	
	}
}

    
/* entrypoint */
function NSGetModule(compMgr, fileSpec) {
    return FireBibleModule;
}