/*
	This file is part of FireBible, a bible study tool for Firefox.
	Copyright (C) 2008 Brian Fernandes <infernalproteus@gmail.com>

    FireBible is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

function FireBible() {
	this.cCount = 0;
	this.name = "chrome://firebible";
	this.bControls = new Array();	
	this.windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
	this.promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                    			.getService(Components.interfaces.nsIPromptService);
    this.observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
    this.observerService.addObserver(this, this.name, false);	
	this.log("Firebible Version 0.8.5");
	this.iTime = new Date();
};

FireBible.prototype.getJSword = function() {
	this.jsword = this.pHandler.getJSword();
	if (!this.jsword) {
		this.jsword = new JSword();
		this.pHandler.setJSword(this.jsword);
		this.pHandler2.setJSword(this.jsword);
		this.jsword.setComponentLocation(this.pHandler.getLocation().parent.path);
	}
	return this.jsword;
};

FireBible.prototype.initializeMenus = function() {
	var prefs = this.getJSword().fbPrefs;
	
	this.initMenuItem(prefs, prefs.VERSE_SEPARATE_LINE);
	this.initMenuItem(prefs, prefs.VERSE_SUPERSCRIPT);
	this.initMenuItem(prefs, prefs.SHOW_HEADINGS);
	this.initMenuItem(prefs, prefs.SHOW_NOTES);
	
	var verseFormat = prefs.getIntPref(prefs.VERSE_FORMAT);
	var menuItem = document.getElementById(prefs.VERSE_FORMAT + "." + verseFormat);
	menuItem.setAttribute("checked", true);
};

FireBible.prototype.initMenuItem = function(prefs, id) {
	var menuItem = document.getElementById(id);
	var checked = prefs.getBoolPref(id);
	menuItem.setAttribute("checked", checked);
};

FireBible.prototype.toggleMenuItem = function(menuItem) {
	var checked = menuItem.getAttribute("checked");	
	checked = (!checked || checked == "false") ? false : true;
	this.jsword.fbPrefs.setBoolPref(menuItem.id, checked);
	this.reload();	
};

FireBible.prototype.setVerseFormat = function(format) {
	this.jsword.fbPrefs.setIntPref(this.jsword.fbPrefs.VERSE_FORMAT, format);
	this.reload();
};

FireBible.prototype.reload = function() {
	if (gBrowser.selectedBrowser.currentURI != null) {
		var uri = gBrowser.selectedBrowser.currentURI.spec;
		if (uri.indexOf("bible://") == 0 || uri.indexOf("sword://") == 0) {
			gBrowser.selectedBrowser.reload();
		}
	}	
};

FireBible.prototype.initFromProtocol = function() {
	this.bibleControl.initBibles();
	//this.bibleControl.selectFirstBible();
};

FireBible.prototype.initBibleList = function(forceReset) {
	var bibles = this.getJSword().getBibleList(forceReset, false);	
	this.bibleControl.bibles = bibles;
	return bibles != null;
};

FireBible.prototype.initBookList = function() {
	this.bibleControl.books = this.getJSword().getBookList();
};

FireBible.prototype.onLoad = function() {
	this.registerProtocol();
	this.toolbarItem = document.getElementById("firebible-toolbarItem");
};

FireBible.prototype.registerProtocol = function() {
	var bibleComponent = Components.classes["@mozilla.org/network/protocol;1?name=bible"];
	this.pHandler = bibleComponent.getService().wrappedJSObject;
	bibleComponent = Components.classes["@mozilla.org/network/protocol;1?name=sword"];
	this.pHandler2 = bibleComponent.getService().wrappedJSObject;
};

FireBible.prototype.initUtils = function() {
	var js = this.getJSword();
	var inited = false;
	if (!js.initializing) {
		js.initializing = true;
		try {	    
			inited = js.initUtils();
		} catch (e) {
			e = js.getRootException(e);
			this.log(e);
			this.printStackTrace(e);
		}
		js.initializing = false;
	} else {
		this.promptService.alert(window, "Info", "JSword is currently being initialized, please try again in a few seconds.");		
	}
	return inited;
};

FireBible.prototype.register = function(element) {
	this.bibleControl = element;
};

FireBible.prototype.onUnload = function() {
};

FireBible.prototype.getGBrowser = function() {
	return gBrowser;
};

FireBible.prototype.log = function(message) {
	if (this.jsword && !this.jsword.fbPrefs.getBoolPref(this.jsword.fbPrefs.DEBUG)) return;
	if (this._logService == null) {
	        this._logService = Components.classes['@mozilla.org/consoleservice;1'].getService();
	        this._logService.QueryInterface(Components.interfaces.nsIConsoleService);
	}
	
	this._logService.logStringMessage("fb: " + message);
};

FireBible.prototype.printStackTrace = function(e) {
	var traces = e.getStackTrace();
	for (var i = 0; i < traces.length; i++) {
		this.log(traces[i].toString());
	}
};

FireBible.prototype.openPrefs = function() {
	if (this.windowManager.getMostRecentWindow("firebible:preferences") == null) {
		window.openDialog('chrome://firebible/content/preferences.xul', '_blank');
	}
};

FireBible.prototype.loadErrorHTML = function(errorMsg) {
	this.jsword.error = errorMsg;
	var url = "sword://status/";
	gBrowser.loadURI(url);		
};

FireBible.prototype.loadHTML = function(bible, book, longName, chapter) {	
	var url = "bible://" + book;
	if (chapter != "ALL") {
		url += "." + chapter + ".1-ff";
	}
	gBrowser.loadURI(url);
};

FireBible.prototype.load = function(uri) {
	var bible = this.bibleControl.bible;
	return this.getJSword().getHTML(bible, unescape(uri.host + uri.path));
};

FireBible.prototype.isIndexed = function(bible) {
	return this.jsword.utilKlass.isIndexed(bible);
};

FireBible.prototype.search = function(bible, search) {
	this.log("searching for " + search); 
	try {
		this.jsword.pageTitle = "Search results for " + search;
		var key = this.jsword.utilKlass.find(bible, search, this.jsword.cl);
		if (!key || key == "") {
			this.loadErrorHTML("No results found");
			return;						
		}
		var uri = "bible://" + key;
		gBrowser.loadURI(uri);
	} catch (e) {
		e = this.jsword.getRootException(e);
		this.promptService.alert(window, "Error", "Error occurred while searching:\n" + e);
		this.log(e);
		this.printStackTrace(e);
	}
};

FireBible.prototype.index = function(bible, stack, bibleControl) {
	this.bibleControl = bibleControl;
	this.progressMeter = stack.firstChild;
	this.indexLabel = stack.lastChild.firstChild;
	this.progressMeter.value = 0;
	this.indexLabel.value = "";
	this.iManager = this.jsword.utilKlass.index(bible);
	this.indexInterval = setInterval("fireBible.checkIndexStatus();", 250);
};

FireBible.prototype.notify = function(message) {
	this.observerService.notifyObservers(this, this.name, message);
};

FireBible.prototype.observe = function(subject, topic, data) {
	if (data == this.getJSword().fbPrefs.MODULE_DIRS) {
		this.log("module directory list modified");
		this.initBibleList(true);
	}
};

FireBible.prototype.checkIndexStatus = function() {
	var details = this.iManager.getIndexStatus();
	this.progressMeter.value = details[1];	//work completed
	this.indexLabel.value = details[2];		//current work section
	var fin;
	if (details[0].getClass) {
		fin = details[0].booleanValue();
	} else {
		fin = details[0];
	}
	if (fin) {
		clearInterval(this.indexInterval);
		this.bibleControl.resetSearch();
		if (details[3] == null && this.isIndexed(this.iManager.getBook())) {
			this.bibleControl.doSearch(null, true);
		} else {
			this.promptService.alert(window, "Error", "The following error occurred while indexing this bible:\n" + this.jsword.getRootException(details[3]));
		}
	}
};

var fireBible = new FireBible();

/*FireBible.prototype.onLoad = function() {
	//window.fireBible = fb;
	fb.onLoad();	
}

function onUnload() {
	fb.onUnload();
	delete fb;
}*/

window.addEventListener("load", function() { fireBible.onLoad(); }, false);
window.addEventListener("unload", function() { fireBible.onUnload(); }, false);