/*
	This file is part of FireBible, a bible study tool for Firefox.
	Copyright (C) 2008 Brian Fernandes <infernalproteus@gmail.com>

    FireBible is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

JSword.prototype.setComponentLocation = function(clocation) {
	this.cLocation = this.getFileURI(clocation);
};

JSword.prototype.initUtils = function() {
	if (this.utilsInitialized) return true;
	this.log("Initializing JSword Utils");
	var installDir = this.fbPrefs.getCharPref(this.fbPrefs.JSWORD_DIR);
	if (!installDir || installDir.length == 0) {
		this.promptService.alert(window, "Error", "JSword directory preference not set.\nPlease set this preference (Tools > FireBible) before proceeding.");
		return false;
	}
	try {
		this.cl = this.getClassLoader(installDir);
		if (!this.cl) return false;
		this.getClassT("org.crosswire.jsword.book.Book");	//try to load this to ensure JSword is around
		this.utilKlass = this.invokeStaticT("com.thegoan.firebible.FBUtils", "getInstance");		
		this.utilKlass.setXSLURI(this.cLocation + "/simple.xsl");
	} catch (e) {
		this.log(e);
		e = this.getRootException(e);
		this.promptService.alert(window, "Error", "JSword could not be loaded from the specified directory.\nPlease correct this location (Tools > FireBible) before proceeding.");
		return false; 
	}
	this.getBibleList(false, true);
	this.getBookList();
	this.utilsInitialized = true;
	this.pageTitle = null;	
	
	return true;
};

JSword.prototype.isInitialized = function() { 
	return this.utilsInitialized;
};

JSword.prototype.getClassLoader = function(jswordDir) {
	var basePath = this.getFileURI(jswordDir) + "/";
	var jarList = new Array();
	jarList[0] = this.cLocation + "/policy/";
	jarList[1] = this.cLocation + "/utils/";
		
	//add all jars in this directory to the classloader
	var file = Components.classes["@mozilla.org/file/local;1"]
                     .createInstance(Components.interfaces.nsILocalFile);
	file.initWithPath(jswordDir);
	if (!file.exists()) {
		this.promptService.alert(window, "Error", "JSword directory specified does not exist.\nPlease correct this location (Tools > FireBible) before proceeding.");
		return null;
	}
	
	var jarRE = /\.jar$/i;	
	var count = 2;
	var entries = file.directoryEntries;
	var str = "";
	while(entries.hasMoreElements()) {
  		var entry = entries.getNext();
  		entry = entry.QueryInterface(Components.interfaces.nsIFile);
  		var fileName = entry.leafName;
  		if (jarRE.test(fileName)) {
  			jarList[count++] = basePath + escape(fileName);
  			str += jarList[count-1] + "\n"; 
  		}  		
	}	
	
	this.log(str);
	try {
		var sClass = java.lang.String;
	} catch (e) {
		this.log(e);
		this.promptService.alert(window, "Error", "Error initiating Java interface.\nPlease ensure that you have Java installed and enabled.\n" + e);
		return null;
	}
	
	var t = new java.net.URL(jarList[0]);
	var urlList = java.lang.reflect.Array.newInstance(t.getClass(), jarList.length);
	for (var i = 0; i < jarList.length; i++) {
		urlList[i] = new java.net.URL(jarList[i]);
	}
	
	var classloader = new java.net.URLClassLoader(urlList);
	//var classloader = java.net.URLClassLoader.newInstance(urlList);
	//setup the policy	
	var cPolicy = java.security.Policy.getPolicy();
	var policyClass = java.lang.Class.forName("com.thegoan.firebible.FBPolicy", true, classloader);
	this.log("Policy class " + policyClass);
	var fbPolicy = policyClass.newInstance();
	fbPolicy.setPolicyURL(cPolicy, urlList);
	java.security.Policy.setPolicy(fbPolicy);
	//java.lang.System.setSecurityManager(null);		
	
	return classloader;
};

JSword.prototype.getSidebarModules = function() {
	return this.utilKlass.getSidebarModules();
};

JSword.prototype.getBibleList = function(forceReset, doNotWarn) {
	if (this.bibleList == null || forceReset) {
		var dirs = this.fbPrefs.getModuleDirs();
		var dirList = null;
		if (dirs != null) { 
			dirList = java.lang.reflect.Array.newInstance(java.lang.Class.forName("java.io.File"), dirs.length);
			for (var i = 0; i < dirs.length; i++) {
				dirList[i] = new java.io.File(dirs[i]);
			}
		}
		var books = this.utilKlass.getBooks(dirList, this.cl);
		var bibleList = this.utilKlass.getBibles();
		if (bibleList.size() == 0) {
			if (!doNotWarn) this.promptService.alert(window, "Warning", "Could not find any Bibles installed.\nPlease setup additional module directories (Tools > FireBible) before proceeding.");
			return null;
		}
		this.bibleList = bibleList;
	}
	return this.bibleList;
};

JSword.prototype.getBookList = function() {
	if (this.bookList == null) {
		var bookList = this.utilKlass.getBooksInfo();
		this.bookList = new Array();
		for (var i = 0; i < bookList.length; i++) {
			this.bookList[i] = new Object();
			this.bookList[i].name = bookList[i][0];
			this.bookList[i].sName = bookList[i][1];
			var cCount = bookList[i][2];
			if (cCount.getClass) {
				this.bookList[i].chapters = cCount.intValue();
			} else {
				this.bookList[i].chapters = cCount;
			}
		}		
	}
	return this.bookList;
};

JSword.prototype.getBookList2 = function() {
	this.log("call for books, book list is " + this.bookList);
	if (this.bookList == null) {
		var tBooks = this.invokeStatic("org.crosswire.jsword.versification.BibleInfo", "booksInBible");
		var bInfoClass = this.getClass("org.crosswire.jsword.versification.BibleInfo");
		var bName = this.getMethod(bInfoClass, "getBookName", java.lang.Integer.TYPE);
		var sbName = this.getMethod(bInfoClass, "getShortBookName", java.lang.Integer.TYPE);
		var chBooks = this.getMethod(bInfoClass, "chaptersInBook", java.lang.Integer.TYPE);
		this.bookList = new Array();
		for (var i = 1; i <= tBooks; i++) {		
			//var nArray = [new java.lang.Integer(i)];
			var nArray = new java.lang.reflect.Array.newInstance(java.lang.Integer, 1);
			nArray[0] = new java.lang.Integer(i);
			var bookName = bName.invoke(null, nArray);
			var sBookName = sbName.invoke(null, nArray);
			var nChapters = chBooks.invoke(null, nArray);
			this.bookList[i-1] = new Object();
			this.bookList[i-1].name = bookName;
			this.bookList[i-1].sName = sBookName;
			this.bookList[i-1].chapters = nChapters;
		}
	}
	return this.bookList;
}

JSword.prototype.getHTML = function(bible, key) {
	try {
		if (key == "status/") {
			return this.utilKlass.getErrorHTML(this.error);
		}
		if (bible == null) {
			//get the default bible
			var defaultBible = this.fbPrefs.getCharPref(this.fbPrefs.DEFAULT_BIBLE);
			if (this.bibleList && defaultBible && defaultBible.length > 0) {
				for (var i = 0; i < this.bibleList.size(); i++) {
					var bibleA = this.bibleList.get(i);
					if (bibleA[0] == defaultBible) {
						bible = bibleA[1];
						break;
					}
				}
			}
		}
		if (bible == null && this.bibleList && this.bibleList.size() > 0) {
			bible = this.bibleList.get(0)[1];
		}
		var html = this.utilKlass.getHTML(bible, key, this.pageTitle, this.fbPrefs.getDisplayPrefs(), this.cl);
		this.pageTitle = null; //clear the title, because we may not always get a chance to set it (if you save the URL for instance)
		return html;
	} catch (e) {
		this.log(this.getRootException(e));
	}
};

JSword.prototype.invokeStatic = function() {
	try {
		return this.invokeStaticT(arguments[0], arguments[1]);
	} catch (e) {
		this.log(this.getRootException(e));
	}		
};

JSword.prototype.invokeStaticT = function() {
	var class = this.getClassT(arguments[0]);
	var method = class.getMethod(arguments[1], []);
	return method.invoke(null, []);
};

JSword.prototype.getClass = function() {
	try {
		return this.getClassT(arguments);
	} catch (e) {
		this.log(this.getRootException(e));
	}
};


JSword.prototype.getClassT = function() {
	var class = java.lang.Class.forName(arguments[0], false, this.cl);
	if (class == null) {	//exceptions don't seem to be thrown automatically with the older plugin
		throw new java.lang.ClassNotFoundException();
	}
	return class;
};

JSword.prototype.getMethod = function() {
	try {
		var class = arguments[0];
		var mName = arguments[1];
		var cSize = arguments.length - 2;
		var mArgs = new java.lang.reflect.Array.newInstance(java.lang.Class, cSize);
		for (var i = 0; i < cSize; i++) {
			mArgs[i] = arguments[i + 2];			
		}
		return class.getMethod(mName, mArgs);		
	} catch (e) {
		this.log(this.getRootException(e));
	}
};

JSword.prototype.getRootException = function(e) {
	while(e != null) {
		if (!e.getClass) {	//with the new Java plugin, e can be a JS string :|
			return e;
		}
		var eClass = e.getClass().getName();
		if (eClass == "java.security.PrivilegedActionException") {
			e = e.getCause();
		} else if (eClass == "java.lang.reflect.InvocationTargetException") {
			e = e.getTargetException();
		} else {
			return e;
		}
	} 
	return null;
};

JSword.prototype.getFileURI = function(path) {
	path = escape(path.replace(/\\/g,'/'));
	return "file://" + ((path.charAt(0) == '/') ? path : ("/" + path));  
};

JSword.prototype.log = function(message) {
	if (!this.fbPrefs.getBoolPref(this.fbPrefs.DEBUG)) return;	
	if (this._logService == null) {
	        this._logService = Components.classes['@mozilla.org/consoleservice;1'].getService();
	        this._logService.QueryInterface(Components.interfaces.nsIConsoleService);
	}
	
	this._logService.logStringMessage("fb: " + message);
};

JSword.prototype.moduleDirsChanged = function() {
	var win = this.windowManager.getMostRecentWindow("navigator:browser");
	if (win != null) win.fireBible.notify(this.fbPrefs.MODULE_DIRS);
};

function JSword() {
	this.promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                    			.getService(Components.interfaces.nsIPromptService);
	this.windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);	
	this.fbPrefs = new firebible_prefs(this);
	this.fbPrefs.initPrefs();
	firebible_prefObserver.register(this.fbPrefs.getPrefBranch(), this);	
	this.log("Loading JSword Utils");
};