/*
	This file is part of FireBible, a bible study tool for Firefox.
	Copyright (C) 2008 Brian Fernandes <infernalproteus@gmail.com>

    FireBible is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

function firebible_prefs(jsword) {
	this.prefBranch = null;
	this.DEBUG = "firebible.debug.active";
	this.JSWORD_DIR = "firebible.jsword.directory";
	this.MODULE_DIRS = "firebible.module.directories";
	this.DEFAULT_BIBLE = "firebible.default.bible";
	//display preferences
	this.VERSE_SEPARATE_LINE = "firebible.verse.newline";
	this.VERSE_SUPERSCRIPT = "firebible.verse.superscript";
	this.VERSE_FORMAT = "firebible.verse.format";
	this.SHOW_HEADINGS = "firebible.show.headings";
	this.SHOW_NOTES = "firebible.show.notes";
	this.jsword = jsword;
	this.getPrefBranch();
};

firebible_prefs.prototype.getPrefBranch = function() {
	if (!this.prefBranch) {
		this.prefBranch = Components.classes['@mozilla.org/preferences-service;1'].getService();
		this.prefBranch = this.prefBranch.QueryInterface(Components.interfaces.nsIPrefBranch);
  	}
  	return this.prefBranch;
};

firebible_prefs.prototype.setCharPref = function(name, value) {
	this.getPrefBranch().setCharPref(name, value);
};

firebible_prefs.prototype.getCharPref = function(name) {
	var value = null;
	try {
		value = this.getPrefBranch().getCharPref(name);
		this.jsword.log("value is " + value);
		if (value && value.length == 0) value = null;		
	} catch (e) {
		this.jsword.log(e);
	}
	return value;	
};

firebible_prefs.prototype.setBoolPref = function(name, value) {
	this.getPrefBranch().setBoolPref(name, value);
};

firebible_prefs.prototype.getBoolPref = function(name) {
	var value = null;
	try {
		value = this.getPrefBranch().getBoolPref(name);
	} catch (e) {
	}
	return value;	
};

firebible_prefs.prototype.setIntPref = function(name, value) {
	this.getPrefBranch().setIntPref(name, value);
};

firebible_prefs.prototype.getIntPref = function(name) {
	var value = null;
	try {
		value = this.getPrefBranch().getIntPref(name);
	} catch (e) {
	}
	return value;	
};

firebible_prefs.prototype.getModuleDirs = function() {
	var dirs = this.getCharPref(this.MODULE_DIRS);
	if (!dirs || dirs.length == 0) return null;
	return dirs.split(';');	
};

firebible_prefs.prototype.getDisplayPrefs = function() {
	var prefString = "";
	if (this.getBoolPref(this.VERSE_SEPARATE_LINE)) {
		prefString += "VLine=true;";	
	}	
	if (!this.getBoolPref(this.VERSE_SUPERSCRIPT)) {
		prefString += "TinyVNum=false;";
	}
	if (!this.getBoolPref(this.SHOW_HEADINGS)) {
		prefString += "Headings=false;";
	}
	if (!this.getBoolPref(this.SHOW_NOTES)) {
		prefString += "Notes=false;"
	}
	var verseFormat = this.getIntPref(this.VERSE_FORMAT);
	switch (verseFormat) {
		case 0:
			prefString += "NoVNum=true;";
			break;
		case 2:
			prefString += "CVNum=true;";
			break;
		case 3:
			prefString += "BCVNum=true;";
	}
	return prefString;
};

firebible_prefs.prototype.initPrefs = function() {
	this.initPref(this.VERSE_SUPERSCRIPT, "bool", true);
	this.initPref(this.SHOW_HEADINGS, "bool", true);
	this.initPref(this.SHOW_NOTES, "bool", true);
	this.initPref(this.VERSE_FORMAT, "int", 1);
};

firebible_prefs.prototype.initPref = function(name, type, dValue) {
	switch(type) {
		case "char":
			var ePref = this.getCharPref(name);
			if (!ePref || ePref.length == 0) {
				this.setCharPref(name, dValue);
			}
			break;
		case "bool":
      		var ePref = this.getBoolPref(name);
      		if (ePref == null) {
        		this.setBoolPref(name, dValue);
        	}
      		break;
      	case "int":
      		var ePref = this.getIntPref(name);
      		if (ePref == null) {
        		this.setIntPref(name, dValue);
        	}
      		break;      	
	}
};


var firebible_prefObserver = {
  register: function(prefBranch, jsword) {
    this._branch = prefBranch;
    this.jsword = jsword;
    this._branch.addObserver("", this, false);
  },

  unregister: function() {
    if(!this._branch) return;
    this._branch.removeObserver("", this);
  },

  observe: function(aSubject, aTopic, aData) {
    if(aTopic != "nsPref:changed") return;
    // aSubject is the nsIPrefBranch we're observing (after appropriate QI)
    // aData is the name of the pref that's been changed (relative to aSubject)
    switch (aData) {
      case this.jsword.fbPrefs.MODULE_DIRS:
      	this.jsword.moduleDirsChanged();
      	break;
    }
  }
};
